@@ -24,6 +24,7 @@
 
 #include "nm-default.h"
 
+#include <locale.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
@@ -35,6 +36,11 @@
 
 #include "utils.h"
 
+#if !GTK_CHECK_VERSION(4,0,0)
+#define gtk_init()			gtk_init(NULL, NULL)
+#define gtk_window_destroy(window)			gtk_widget_destroy(GTK_WIDGET (window))
+#endif
+
 #define KEYRING_UUID_TAG "connection-uuid"
 #define KEYRING_SN_TAG "setting-name"
 #define KEYRING_SK_TAG "setting-key"
@@ -203,7 +209,8 @@ eui_finish (const char *vpn_name,
 	                        need_proxypass && allow_interaction);
 
 	keyfile_add_entry_info (keyfile,
-	                        NM_OPENVPN_KEY_CHALLENGE_RESPONSE,
+	                        need_challengeresponse_echo ? NM_OPENVPN_HINT_CHALLENGE_RESPONSE_ECHO
+	                                                    : NM_OPENVPN_HINT_CHALLENGE_RESPONSE_NOECHO,
 	                        "",
 	                        _("Challenge response"),
 	                        need_challengeresponse_echo,
@@ -243,7 +250,7 @@ std_ask_user (const char *vpn_name,
 	g_return_val_if_fail (out_new_certpass != NULL, FALSE);
 	g_return_val_if_fail (out_new_proxypass != NULL, FALSE);
 
-	gtk_init (NULL, NULL);
+	gtk_init ();
 
 	dialog = NMA_VPN_PASSWORD_DIALOG (nma_vpn_password_dialog_new (_("Authenticate VPN"), prompt, NULL));
 
@@ -276,7 +283,7 @@ std_ask_user (const char *vpn_name,
 		success = TRUE;
 	}
 
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_window_destroy (GTK_WINDOW (dialog));
 	return success;
 }
 
@@ -486,7 +493,6 @@ main (int argc, char *argv[])
 	NoSecretsRequiredFunc no_secrets_required_func;
 	AskUserFunc ask_user_func;
 	FinishFunc finish_func;
-
 	GOptionContext *context;
 	GOptionEntry entries[] = {
 			{ "reprompt", 'r', 0, G_OPTION_ARG_NONE, &retry, "Reprompt for passwords", NULL},
@@ -498,15 +504,23 @@ main (int argc, char *argv[])
 			{ "hint", 't', 0, G_OPTION_ARG_STRING_ARRAY, &hints, "Hints from the VPN plugin", NULL},
 			{ NULL }
 		};
+	GError *error = NULL;
 
+	setlocale (LC_ALL, "");
 	bindtextdomain (GETTEXT_PACKAGE, NULL);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
 	context = g_option_context_new ("- openvpn auth dialog");
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+#if !GTK_CHECK_VERSION(4,0,0)
 	g_option_context_add_group (context, gtk_get_option_group (FALSE));
-	g_option_context_parse (context, &argc, &argv, NULL);
+#endif
+	if (!g_option_context_parse (context, &argc, &argv, &error)) {
+		fprintf (stderr, "Error parsing options: %s\n", error->message);
+		g_error_free (error);
+		return 1;
+	}
 	g_option_context_free (context);
 
 	if (vpn_uuid == NULL || vpn_name == NULL || vpn_service == NULL) {
